# coding: utf-8

require 'base64'

# ========================================================================
# 管理者クラス.
class AdminController < ApplicationController

  skip_before_action :verify_authenticity_token

  # アクセス制限
  before_filter :block_foreign_hosts
  # Basic認証定義.
  before_filter :auth

  # ----------------------------------------------------------------------
  # Basic認証実装.
  # パスワードはSHA-1ハッシュ値HEXで覚える.
  def auth
    name = SHIBCERT_CONFIG[Rails.env]['admin_basic_name']
    passwd = SHIBCERT_CONFIG[Rails.env]['admin_basic_pswd']
    authenticate_or_request_with_http_basic do |user, pass|
      user == name && Digest::SHA1.hexdigest(pass) == passwd
    end
  end

  # ----------------------------------------------------------------------
  # トップ画面.
  def index

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    # ページングサイズ
    limit = SHIBCERT_CONFIG['flag']['admin_page_num']

    # 初期設定
    @uid_ck = false
    @name_ck = false
    @mail_ck = false
    @totop = false

    # メンテナンスモード.
    @mmode = false
    if maintenance?
      @mmode = true
      flash[:alert] = maintenance_mesg
    end

    if params[:key].blank?
      # 全て: ページング
      @uid_ck = true
      count = User.count
      if count <= 0
        errorMesg('No user.')
        return
      end
      @page_max = count / limit
      if @page_max * limit != count
        @page_max += 1
      end
      @page_num = params[:page] == nil ? 1 : params[:page].to_i
      if @page_num < 1
        @page_num = 1
      elsif @page_num > @page_max
        @page_num = @page_max
      end
      if @page_num > 1
        @totop = true
      end
      @users = User.all.order("updated_at DESC").limit(limit).offset(limit*(@page_num-1))
      @search_num = @users.length
    else
      # 検索
      @key = params[:key]
      if params[:admin][:opt] == "uid"
        item = "uid"
        @uid_ck = true
      elsif params[:admin][:opt] == "name"
        item = "name"
        @name_ck = true
      else
        item = "email"
        @mail_ck = true
      end
      @page_max = 0
      @page_num = 0
      @users = User.where(item + " like '%" + params[:key] + "%'").order("updated_at DESC")
      @search_num = @users.length
      if @search_num > limit
        @users.slice!(limit-1..@search_num-limit)
      end
    end

  end

  # ----------------------------------------------------------------------
  # ユーザ情報.
  def user

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    # 引数確認
    if params[:id].blank?
      errorMesg('Argument error.')
      return
    end

    # メンテナンスモード.
    @mmode = false
    if maintenance?
      @mmode = true
      flash[:alert] = maintenance_mesg
    end

    # ユーザ検索
    @user = User.find_by_id(params[:id])
    if @user.blank?
      errorMesg('Search user error (' + params[:id] + ').')
      return
    end

    # 証明書情報
    @certs = Cert.where(user_id: @user.id).order("created_at DESC")

  end

  # ----------------------------------------------------------------------
  # 証明書情報.
  def cert

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    # 引数確認
    if params[:id].blank?
      errorMesg('Argument error.')
      return
    end

    # メンテナンスモード.
    @mmode = false
    if maintenance?
      @mmode = true
      flash[:alert] = maintenance_mesg
    end

    # 証明書情報
    @cert = Cert.find_by_id(params[:id])
    if @cert.blank?
      errorMesg('Search certificate error (' + params[:id] + ').')
      return
    end

  end

  # ----------------------------------------------------------------------
  # メンテナンスモード設定画面.
  def mainte

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    # メンテナンスモード.
    flash[:alert] = nil
    @mmode = false
    if maintenance?
      # メンテナンス中.
      @mmode = true
      flash[:alert] = maintenance_mesg
    else
      # メンテナンス中文字列取得.
      @jmesg = nil
      @emesg = nil
      yml_def = YAML.load_file("config/maintenance.yml.default")
      if !yml_def.blank?
        @jmesg = yml_def["ja"]
        @emesg = yml_def["en"]
      end
      if @jmesg.blank?
        @jmesg = "メンテナンス中。"
      end
      if @emesg.blank?
        @emesg = "under maintenance."
      end
    end

  end

  # ----------------------------------------------------------------------
  # メンテナンスモード設定の実行.
  def mainte_set

    if params[:mainte][:mode] == "cancel"
      # メンテナンス解除(ファイルの削除).
      File.delete "config/maintenance.yml"
    else
      # メンテナンスセット(ファイルの作成).
      out = File.open('config/maintenance.yml', 'w')
      out.puts("ja: " + params[:mainte][:jmesg])
      out.puts("en: " + params[:mainte][:emesg])
      out.close()
    end

    # メンテナンス画面へ.
    redirect_to :action => "mainte"

  end

  # ----------------------------------------------------------------------
  # TSV解析
  def parseTsv(tsv)

    tsvs = { wait:[], issue:[], update:[], revoke:[], error:[] }
    tsv.each { |cert|
      # TSV情報解析
      err = false
      # シリアル番号で検索(必ず1つだけ).
      if !cert["発行・更新申請-シリアル番号"].blank?
        mycert = Cert.find_by_serialnumber(cert["発行・更新申請-シリアル番号"])
      else
        err = true      # シリアル番号が無いものはエラーとする(2020-03).
      end
      # DNは更新時に重複する可能性があるので使わない
#      if mycert.blank? && !cert["主体者DN"].blank?
#        mycert = Cert.find_by_dn(cert["主体者DN"])
#      end
      if !mycert.blank?
        cert["DB登録"] = "登録済み"
      end
      if !err
        # エラーじゃない.
        case cert["状態"].to_i
        # 4:発行案内メール送信済み(取得待ち)
        when 4 then
          tsvs[:wait].push cert
        # 5:証明書発行処理完了
        when 5 then
          tsvs[:issue].push cert
        # 6:証明書更新処理完了
        when 6 then
          tsvs[:update].push cert
        # 7:証明書失効処理完了
        when 7 then
          tsvs[:revoke].push cert
        else
          tsvs[:error].push cert
        end
      else
        # シリアル番号が無い(DB的に1つとは限らないのでエラー).
        tsvs[:error].push cert
      end
    }

    return tsvs

  end

  # ----------------------------------------------------------------------
  # NIIサーバ同期.
  def sync

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    # TSVファイル取得
    tsv = RaReq.requestAll();
    if tsv.blank?
      errorMesg('Download server certificate info error.')
      return
    end

    # TSV解析
    @tsvs = parseTsv(tsv)
    if @tsvs.blank?
      errorMesg('Download server certificate info parse error.')
      return
    end

    # カウント
    @count = tsv.length
    @upcount = (@tsvs[:wait].select {|val| val["DB登録"] == "登録済み" }).length \
             + (@tsvs[:issue].select {|val| val["DB登録"] == "登録済み" }).length \
             + (@tsvs[:update].select {|val| val["DB登録"] == "登録済み" }).length \
             + (@tsvs[:revoke].select {|val| val["DB登録"] == "登録済み" }).length \
             + (@tsvs[:error].select {|val| val["DB登録"] == "登録済み" }).length

  end

  # ----------------------------------------------------------------------
  # 取得証明書の表示.
  def show

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    # 情報取得(ファイルから)
    tsv = RaReq.existAll();
    if tsv.blank?
      errorMesg('Download server certificate info error.')
      return
    end

    # TSV解析
    tsvs = parseTsv(tsv)
    if tsvs.blank?
      errorMesg('Download server certificate info parse error.')
      return
    end

    # 種別
    type = 0
    @type_name = "unknown"
    if !params[:type].blank?
      type = params[:type].to_i
    end

    # セット
    @certs = []
    case type
    when 4 then
        @type_name = t('admin.sync.wait')
        @certs = tsvs[:wait]
    when 5 then
        @type_name = t('admin.sync.issue')
        @certs = tsvs[:issue]
    when 6 then
        @type_name = t('admin.sync.update')
        @certs = tsvs[:update]
    when 7 then
        @type_name = t('admin.sync.revoke')
        @certs = tsvs[:revoke]
    else
        @type_name = t('admin.sync.error')
        @certs = tsvs[:error]
    end

  end

  # ----------------------------------------------------------------------
  # DB登録実行
  def db_update(certs)

    count = 0
    certs.each { |cert|
      # ユーザ情報.
      email = cert['利用者mail']
      name = cert['利用者氏名']
      user = User.find_by_email(email) || User.create_from_tsv(email, name)
      user_id = 0
      if user != nil
        user_id = user.id
      end
      # 証明書情報
      serial = cert['発行・更新申請-シリアル番号']
      dn = cert['主体者DN']
      purpose_type = cert['証明書プロファイルID']
      purpose_type = !purpose_type || purpose_type.to_i
      nii_state = cert['状態']
      state = 0
      if !nii_state.blank?
        # shib-cert用にステータス変換.
        nii_state = nii_state.to_i
        state = get_state(nii_state)
      end
      memo = cert['発行・更新申請-申請ID']
      get_at = cert['発行・更新申請-処理完了日時']
      if get_at.blank?
        get_at = cert['発行・更新申請-日時']
      end
      expire_at = cert['発行・更新申請-証明書有効期限']
#      limit_at = cert['URL期限']  # 未使用.
      download_type = cert['ダウンロード方法']
      download_type = !download_type || download_type.to_i
      revoke_reason = cert['失効申請-失効理由コード']
      revoke_reason = !revoke_reason || revoke_reason.to_i
      revoke_comment = cert['失効申請-失効理由コメント']
      # DB証明書の取得.
      mycert = nil
      if !serial.blank?
        # シリアル番号で検索(必ず1つだけ).
        mycert = Cert.find_by_serialnumber(serial)
      end
      # DNは更新時に重複するので使わない.
#      if mycert.blank? && !dn.blank?
#        mycert = Cert.find_by_dn(dn)
#      end
      # 取得した値のセット.
      if mycert.blank?
        # 新規登録.
        ActiveRecord::Base.transaction do
          user.cert_serial_max += 1
          user.save!
        end
        mycert = Cert.new
        mycert.req_seq = user.cert_serial_max
        mycert.user_id = user_id
        mycert.serialnumber = serial
        mycert.dn = dn
        mycert.purpose_type = purpose_type
        mycert.state = state
        mycert.memo = "申請:" + memo
        if !get_at.blank?
          mycert.get_at = Date.parse(get_at)
        end
        if !expire_at.blank?
          mycert.expire_at = Date.parse(expire_at)
        end
        mycert.download_type = download_type
        mycert.revoke_reason = revoke_reason
        mycert.revoke_comment = revoke_comment
      else
        # 更新.
        mycert.user_id ||= user_id                  # 未設定時のみ上書き.
        mycert.serialnumber ||= serial              # 未設定時のみ上書き.
        mycert.dn ||= dn                            # 未設定時のみ上書き.
        mycert.purpose_type ||= purpose_type        # 未設定時のみ上書き.
        mycert.state = state                        # 状態は必ず上書き.
        mycert.memo = "申請:" + memo                # 申請番号で上書き.
        if mycert.get_at.blank? && !get_at.blank?
          mycert.get_at = Date.parse(get_at)        # 未設定時のみ上書き.
        end
        if !expire_at.blank?
          mycert.expire_at = Date.parse(expire_at)  # あれば上書き.
        end
        mycert.download_type ||= download_type      # 未設定時のみ上書き.
        mycert.revoke_reason ||= revoke_reason      # 未設定時のみ上書き.
        mycert.revoke_comment ||= revoke_comment    # 未設定時のみ上書き.
      end
      # 保存.
      mycert.save!
      count += 1 
    }
    return count

  end

  # ステータス変換.
  def get_state(state)
    new_state = 0
    case state
    # 0:証明書発行処理中
    when 0
      new_state = Cert::State::NEW_REQUESTED_TO_NII
    # 1:証明書更新処理中
    when 1
      new_state = Cert::State::RENEW_REQUESTED_TO_NII
    # 2:証明書失効処理中
    when 2
      new_state = Cert::State::REVOKE_REQUESTED_TO_NII
    # 3:発行案内メール送信待ち
    when 3
      new_state = Cert::State::NEW_REQUESTED_TO_NII
#      new_state = Cert::State::NEW_ERROR
    # 4:発行案内メール送信済み
    when 4
      new_state = Cert::State::NEW_REQUESTED_TO_NII
#      new_state = Cert::State::NEW_ERROR
    # 5:証明書発行処理完了
    when 5
      new_state = Cert::State::NEW_GOT_SERIAL
    # 6:証明書更新処理完了
    when 6
      new_state = Cert::State::RENEW_GOT_SERIAL
    # 7:証明書失効処理完了.
    when 7
      new_state = Cert::State::REVOKED
    # 8:新証明書への置き換え待ち
    when 8
      new_state = Cert::State::UNKNOWN
    # 9:発行・更新完了メール送信待ち
    when 9
      new_state = Cert::State::NEW_REQUESTED_TO_NII
    # 10:失効完了メール送信待ち
    when 10
      new_state = Cert::State::REVOKE_REQUESTED_TO_NII
    # その他:不明.
    else
      new_state = Cert::State::UNKNOWN
    end
    return new_state
  end
=begin
  申請の状態
  0:証明書発行処理中
  1:証明書更新処理中
  2:証明書失効処理中
  3:発行案内メール送信待ち
  4:発行案内メール送信済み
  5:証明書発行処理完了
  6:証明書更新処理完了
  7:証明書失効処理完了
  8:新証明書への置き換え待ち
  9:発行・更新完了メール送信待ち
  10:失効完了メール送信待ち
  -1:フォーマットエラー
  -2:認証局連携API処理エラー
  -3:支援システム実行エラー
=end

  # ----------------------------------------------------------------------
  # 取得証明書のDB反映
  def update_post

    if !current_user
      return redirect_to :action => "index"
    end

    # 情報取得(ファイルから)
    tsv = RaReq.existAll();
    if tsv.blank?
      flash[:alert] = 'Download server certificate info error.'
      redirect_to :action => "cert" and return
    end

    # TSV解析
    tsvs = parseTsv(tsv)
    if tsvs.blank?
      flash[:alert] = 'Download server certificate info parse error.'
      redirect_to :action => "cert" and return
    end

    count = 0;

    # 取得待ち
    if params[:import][:wait_new] != "0"
      certs = tsvs[:wait].select {|val| val["DB登録"] != "登録済み" }
      num = db_update(certs)
      count += num
    end
    if params[:import][:wait_exist] != "0"
      certs = tsvs[:wait].select {|val| val["DB登録"] == "登録済み" }
      num = db_update(certs)
      count += num
    end

    # 発行済み
    if params[:import][:issue_new] != "0"
      certs = tsvs[:issue].select {|val| val["DB登録"] != "登録済み" }
      num = db_update(certs)
      count += num
    end
    if params[:import][:issue_exist] != "0"
      certs = tsvs[:issue].select {|val| val["DB登録"] == "登録済み" }
      num = db_update(certs)
      count += num
    end

    # 更新済み
    if params[:import][:update_new] != "0"
      certs = tsvs[:update].select {|val| val["DB登録"] != "登録済み" }
      num = db_update(certs)
      count += num
    end
    if params[:import][:update_exist] != "0"
      certs = tsvs[:update].select {|val| val["DB登録"] == "登録済み" }
      num = db_update(certs)
      count += num
    end

    # 失効済み
    if params[:import][:revoke_new] != "0"
      certs = tsvs[:revoke].select {|val| val["DB登録"] != "登録済み" }
      num = db_update(certs)
      count += num
    end
    if params[:import][:revoke_exist] != "0"
      certs = tsvs[:revoke].select {|val| val["DB登録"] == "登録済み" }
      num = db_update(certs)
      count += num
    end

=begin
    # エラー(登録しない).
    if params[:import][:error_new] != "0"
      certs = tsvs[:error].select {|val| val["DB登録"] != "登録済み" }
      num = db_update(certs)
      count += num
    end
    if params[:import][:error_exist] != "0"
      certs = tsvs[:error].select {|val| val["DB登録"] == "登録済み" }
      num = db_update(certs)
      count += num
    end
=end

    # DB登録処理結果
    if count > 0
      flash[:notice] = "DataBase Updated (" + count.to_s + ") Certificates."
    else
      flash[:alert] = "DataBase No Updated (may be not seleced)."
    end

    # 取得ファイルの削除
    RaReq.clearAll();

    # 管理者トップ画面に戻る
    redirect_to :action => "index"

  end

  # ----------------------------------------------------------------------
  # 取得証明書の表示.
  def clear

    if !current_user
      return redirect_to :action => "index"
    end

    # 取得ファイルの削除
    RaReq.clearAll();

    # 管理者トップ画面に戻る
    redirect_to :action => "index"

  end

  # ----------------------------------------------------------------------
  # 証明書エラー表示.
  def error

    if !current_user
      return redirect_to controller: :certs, :action => "index"
    end

    if params[:id].blank?
      errorMesg('cert_id undefined.')
      return
    end

#    errorMesg('error test.')
#    return

    # 証明書エラーファイルの確認
    html = nil
    filename = "log/cert_" + params[:id] + "_error.html"
    if !File.exist?(filename)
      errorMesg('cert_error_file not found.')
      return
    end

    # 証明書エラーファイルの読み込み
    open(filename) do |fp|
      html = fp.read
    end
    if html.blank?
      errorMesg('cert_error_file cannot read.')
      return
    end

    # HTML表示
    send_data html, type: 'text/html', disposition: :inline

  end

  # ----------------------------------------------------------------------
  # エラー表示.
  def errorMesg(mesg)
    @mesg = mesg.html_safe if !mesg.blank?
    render action: 'errorMesg'
  end

  # ----------------------------------------------------------------------
  # 失効申請
  def disable_post

    if !current_user
      return redirect_to :action => "index"
    end

#    flash[:alert] = "revoke error"
#    redirect_to :action => "cert" and return

    if params[:id].blank?
      flash[:alert] = "cert_id undefined."
      redirect_to :action => "cert" and return
    end

    # 証明書取得
    @cert = Cert.find_by_id(params[:id])
    if @cert.blank?
      flash[:alert] = "cert undefined."
      redirect_to :action => "cert" and return
    end

    # 画面表示確認用
#    redirect_to action:"disable_result", id:@cert.id and return

    # 証明書ステータスセット
    @cert.state = Cert::State::REVOKE_REQUESTED_FROM_USER
    @cert.save

    # リクエスト発行
    Rails.logger.debug "RaReq.request call: @cert = #{@cert.inspect}"
    RaReq.request(@cert)

    # 完了表示
    redirect_to action:"disable_result", id:@cert.id

  end

  # ----------------------------------------------------------------------
  # 失効申請完了画面
  def disable_result

    if !current_user
      return redirect_to :action => "index"
    end

    @cert = Cert.find_by_id(params[:id])

  end

  # ----------------------------------------------------------------------
  # 証明書削除操作
  def delete_post

    if !current_user
      return redirect_to :action => "index"
    end

    # 画面表示確認用
#    @cert = Cert.find_by_id(params[:id])
#    redirect_to action:"delete_result", id:@cert.user_id and return

    if params[:id].blank?
      flash[:alert] = "cert_id undefined."
      redirect_to :action => "cert" and return
    end

    # 証明書取得
    @cert = Cert.find_by_id(params[:id])
    if @cert.blank?
      flash[:alert] = "cert undefined."
      redirect_to :action => "cert" and return
    end

    # 画面表示確認用
#    redirect_to action:"delete_result", id:@cert.user_id and return

    # エラーファイル削除
    filename = "log/cert_" + params[:id] + "_error.html"
    if File.exist?(filename)
      File.delete filename
    end

    # DB削除
    @user_id = @cert.user_id
    Rails.logger.debug "Cert record delete: @cert = #{@cert.inspect}"
    @cert.destroy

    # 完了表示
    redirect_to action:"delete_result", id:@user_id

  end

  # ----------------------------------------------------------------------
  # 証明書削除結果
  def delete_result

    if !current_user
      return redirect_to :action => "index"
    end

    @user_id = params[:id]

  end

  # ----------------------------------------------------------------------
  # 管理者チェック
  def self.isAdmin(user)
    if user.blank? || user.uid.blank? 
      return false
    elsif user.uid == SHIBCERT_CONFIG[Rails.env]['admin_uid1']   # 設定1
      return true
    elsif user.uid == SHIBCERT_CONFIG[Rails.env]['admin_uid2']   # 設定2
      return true
    elsif user.admin == true    # データベース設定フラグ(現在未使用)
      return false
#      return true
    end
    return false
  end

end
# ========================================================================
